/*
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.inteligr8.wildfly.maven;

import java.util.List;

import org.eclipse.aether.graph.DependencyFilter;
import org.eclipse.aether.graph.DependencyNode;

public class ModuleProvidedDependencyFilter implements DependencyFilter {
    
    private final DependencyAwareComponent dac;
    private final DependencyIndex dindex;
    
    public ModuleProvidedDependencyFilter(DependencyAwareComponent dac, DependencyIndex dindex) {
        this.dac = dac;
        this.dindex = dindex;
    }
    
    @Override
    public boolean accept(DependencyNode node, List<DependencyNode> parents) {
        String nodeId = this.dindex.toId(node);
        
        if (parents == null || parents.isEmpty()) {
            // root element; the project itself
            return true;
        }
        
        switch (this.dindex.getType(nodeId)) {
            case InSystemModule:
            case InModule:
                if (this.dac.getLog().isDebugEnabled())
                    this.dac.getLog().debug(nodeId + ": is provided by a module");
                node.setScope("provided");
                break;
            default:
        }
        
        return true;
    }

}
