/*
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.inteligr8.wildfly.maven;

import java.util.List;

import org.eclipse.aether.graph.DependencyFilter;
import org.eclipse.aether.graph.DependencyNode;

import com.inteligr8.wildfly.maven.DependencyIndex.DependencyType;

public class ValidationDependencyFilter implements DependencyFilter {
    
    private final DependencyAwareComponent dac;
    private final DependencyIndex dindex;
    
    public ValidationDependencyFilter(DependencyAwareComponent dac, DependencyIndex dindex) {
        this.dac = dac;
        this.dindex = dindex;
    }
    
    @Override
    public boolean accept(DependencyNode node, List<DependencyNode> parents) {
        if (parents == null || parents.isEmpty()) {
            // root element; the project itself
            return true;
        }

        for (DependencyNode parent : parents) {
            String parentId = this.dindex.toId(parent);
        
            DependencyType type = this.dindex.getType(parentId);
            if (type == null) {
                if (parent.getDependency() == null) {
                    this.dac.getLog().warn("Has no dependency information: " + parent.getArtifact() + " => " + node.getArtifact());
                } else {
                    switch (parent.getDependency().getScope()) {
                        case "test":
                            this.dac.getLog().debug("The dependency parent was excluded as a test artifact: " + parent);
                        default:
                    }
                }

                continue;
            }
            
            switch (type) {
                case Module:
                case SystemModule:
                    if (this.dac.getLog().isDebugEnabled())
                        this.dac.getLog().debug(parentId + " is a module");
                    return true;
                default:
                    // ignore if the node isn't a module
                    return true;
            }
        }

        this.dac.getLog().error("None of the dependency parents were processed before the artifact: " + node.getDependency());
        return false;
    }

}
