/*
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.inteligr8.wildfly.maven.goal;

import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;
import org.codehaus.plexus.component.annotations.Component;
import org.codehaus.plexus.util.FileUtils;
import org.eclipse.aether.graph.Dependency;

import com.inteligr8.wildfly.maven.model.WildflyModule;

/**
 * @author brian@inteligr8.com
 */
@Mojo( name = "deployment-copy-dependencies", threadSafe = false, defaultPhase = LifecyclePhase.PREPARE_PACKAGE )
@Component( role = org.apache.maven.plugin.Mojo.class )
public class CopyDeploymentResourcesGoal extends AbstractDeploymentGoal {

    @Parameter( property = "wildfly.deployment.ear.libPath", required = true, defaultValue = "lib" )
    protected String libPath;

    private File libDirectory;
    
    private List<String> manifestClassPath = new LinkedList<>();
    
    @Override
    protected void executePre() throws MojoExecutionException {
        if (this.isWebApp()) {
            File webinfDirectory = this.directoryAccessor.validateAndInitializeDirectory(this.outputDirectory, "WEB-INF");
            this.libDirectory = this.directoryAccessor.validateAndInitializeDirectory(webinfDirectory, "lib");
        } else {
            this.libDirectory = this.directoryAccessor.validateAndInitializeDirectory(this.outputDirectory, this.libPath);
        }
    }
    
    @Override
    protected void handleDependency(Dependency dependency) throws MojoExecutionException {
        try {
            FileUtils.copyFileToDirectory(dependency.getArtifact().getFile(), this.libDirectory);
            this.manifestClassPath.add(this.libPath + "/" + dependency.getArtifact().getFile().getName());
        } catch (IOException ie) {
            throw new MojoExecutionException("An I/O related issue occurred", ie);
        }

        if (this.getLog().isInfoEnabled())
            this.getLog().info("Copied resource artifact: " + dependency.getArtifact());
    }
    
    @Override
    protected void handleModuleDependency(Dependency dependency, WildflyModule module) {
    }
    
    @Override
    public void executePost() throws MojoExecutionException {
        
    }

}
