/*
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.inteligr8.wildfly.maven.goal;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.codehaus.plexus.component.annotations.Component;
import org.eclipse.aether.graph.Dependency;
import com.inteligr8.wildfly.maven.model.WildflyModule;
import com.inteligr8.wildfly.maven.xml.DeploymentXmlBuilder;

/**
 * @author brian@inteligr8.com
 */
@Mojo( name = "deployment-generate-xml", threadSafe = false, defaultPhase = LifecyclePhase.GENERATE_RESOURCES )
@Component( role = org.apache.maven.plugin.Mojo.class )
public class GenerateDeploymentXmlGoal extends AbstractDeploymentGoal {
    
    private DeploymentXmlBuilder deploymentXmlBuilder;
    
    @Override
    protected void executePre() throws MojoExecutionException {
        try {
            this.deploymentXmlBuilder = new DeploymentXmlBuilder(this.deploymentSchemaVersion);
        } catch (ParserConfigurationException pce) {
            throw new MojoExecutionException("This should never happen", pce);
        }
    }
    
    @Override
    protected void handleDependency(Dependency dependency) throws MojoExecutionException {
    }
    
    @Override
    protected void handleModuleDependency(Dependency dependency, WildflyModule module) {
        if (this.getLog().isInfoEnabled())
            this.getLog().info("Detected module dependency; adding reference to deployment: " + dependency.getArtifact() + " => " + module.id);
        this.deploymentXmlBuilder.addDependency(module, true);
    }
    
    @Override
    public void executePost() throws MojoExecutionException {
        this.writeDeploymentXml();
    }
    
    private void writeDeploymentXml() throws MojoExecutionException {
        File directory = this.validateAndInitializeDirectory(this.isWebApp() ? "WEB-INF" : "META-INF");
        
        try {
            this.deploymentXmlBuilder.writeTo(directory);
        } catch (TransformerException te) {
            throw new MojoExecutionException("An XML to file transformation issue occurred", te);
        } catch (IOException ie) {
            throw new MojoExecutionException("An I/O issue occurred", ie);
        }
    }

}
