/*
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.inteligr8.wildfly.maven.goal;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.codehaus.plexus.component.annotations.Component;
import org.eclipse.aether.artifact.DefaultArtifact;
import org.eclipse.aether.graph.Dependency;

import com.inteligr8.wildfly.maven.model.WildflyModule;
import com.inteligr8.wildfly.maven.xml.ModuleXmlBuilder;

/**
 * @author brian@inteligr8.com
 */
@Mojo( name = "module-generate-xml", threadSafe = false, defaultPhase = LifecyclePhase.GENERATE_RESOURCES )
@Component( role = org.apache.maven.plugin.Mojo.class )
public class GenerateModuleXmlGoal extends AbstractModuleGoal {
	
    private ModuleXmlBuilder moduleXmlBuilder;
    
    @Override
    protected void executePre() throws MojoExecutionException {
        try {
            this.moduleXmlBuilder = new ModuleXmlBuilder(this.moduleId, this.moduleSchemaVersion, this.moduleVersion);
        } catch (ParserConfigurationException pce) {
            throw new MojoExecutionException("This should never happen", pce);
        }
        
        if (!"pom".equals(this.project.getPackaging()))
            this.moduleXmlBuilder.addResource(new DefaultArtifact(this.project.getArtifact().getId()));
    }
    
    @Override
    protected void handleDependency(Dependency dependency) throws MojoExecutionException {
        if (this.getLog().isDebugEnabled())
            this.getLog().debug("Determined to be resource: " + dependency.getArtifact());
        this.moduleXmlBuilder.addResource(dependency.getArtifact());
    }
    
    @Override
    protected void handleModuleDependency(Dependency dependency, WildflyModule module) throws MojoExecutionException {
        if (this.getLog().isDebugEnabled())
            this.getLog().debug("Determined to be module: " + dependency.getArtifact() + " => " + module.id);
        this.moduleXmlBuilder.addDependency(module, false);
    }
    
    @Override
    public void executePost() throws MojoExecutionException {
        File outputDirectory = this.validateAndInitializeDirectory();
        
        try {
            this.moduleXmlBuilder.writeTo(outputDirectory);
        } catch (TransformerException te) {
            throw new MojoExecutionException("An XML to file transformation issue occurred", te);
        } catch (IOException ie) {
            throw new MojoExecutionException("An I/O issue occurred", ie);
        }
    }

}
