/*
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.inteligr8.wildfly.maven.xml;

import javax.xml.parsers.ParserConfigurationException;

import org.codehaus.plexus.util.StringUtils;
import org.eclipse.aether.artifact.Artifact;
import org.w3c.dom.Element;

/**
 * {@link https://jboss-modules.github.io/jboss-modules/manual/}
 * @author brian@inteligr8.com
 */
public class ModuleXmlBuilder extends AbstractXmlBuilder {

    private final Element resourcesElement;
    
    public ModuleXmlBuilder(String id, String schemaVersion) throws ParserConfigurationException {
        this(id, schemaVersion, null);
    }
    
    public ModuleXmlBuilder(String id, String schemaVersion, String moduleVersion) throws ParserConfigurationException {
        super("urn:jboss:module:" + schemaVersion, schemaVersion.equals("1.0") ? true : false);

        this.resourcesElement = this.xmldoc.createElementNS(this.ns, "resources");

        String name = id;
        if (!this.enableSlotAttribute && moduleVersion != null && !moduleVersion.equals("main"))
            name += ":" + moduleVersion;
        
        Element moduleElement = this.xmldoc.createElementNS(this.ns, "module");
        moduleElement.setAttribute("name", name);
        if (this.enableSlotAttribute && moduleVersion != null && !moduleVersion.equals("main"))
            moduleElement.setAttribute("slot", moduleVersion);
        moduleElement.appendChild(this.resourcesElement);
        moduleElement.appendChild(this.dependenciesElement);
        
        this.xmldoc.appendChild(moduleElement);
    }
    
    @Override
    public String getXmlFilename() {
        return "module.xml";
    }
    
    public void addResource(Artifact artifact) {
        StringBuilder filename = new StringBuilder(artifact.getArtifactId())
                .append('-').append(artifact.getVersion());
        if (StringUtils.isNotEmpty(artifact.getClassifier()))
            filename.append('-').append(artifact.getClassifier());
        filename.append('.').append(artifact.getExtension());
        
        Element resourceRootElement = this.xmldoc.createElementNS(this.ns, "resource-root");
        resourceRootElement.setAttribute("path", filename.toString());
        this.resourcesElement.appendChild(resourceRootElement);
    }

}
